import React, { useEffect, useRef, useCallback, useMemo } from "react";

import type { SideNavigationItem, SideNavigationMenuProps } from "./types";

import * as events from "devextreme/events";

import clients from "../../../../../public/images/noun-support-186489.png";
import activityMonitor from "../../../../../public/images/noun-worker-4615779.png";
import workbench from "../../../../../public/images/noun-work-1612352.png";
import controlPanel from "../../../../../public/images/noun-control-6465754.png";
import digitalService from "../../../../../public/images/noun-digital-2995238.png";
import dataStudio from "../../../../../public/images/noun-complex-data-6195099.png";
import administration from "../../../../../public/images/noun-management-5325385.png";
import { TreeView } from "devextreme-react/tree-view";
import { useScreenSize } from "polaris-ui-foundation";
import { useNavigation } from "../../../../context/navigation";

const getCustomIcon = (icon: string): any => {
  switch (icon) {
    case "clients":
      return clients;
    case "activityMonitor":
      return activityMonitor;
    case "workbench":
      return workbench;
    case "controlPanel":
      return controlPanel;
    case "digitalService":
      return digitalService;
    case "dataStudio":
      return dataStudio;
    case "administration":
      return administration;
    default:
      return icon;
  }
};

export const SideNavigationMenu = (
  props: React.PropsWithChildren<SideNavigationMenuProps>
) => {
  const {
    navigation,
    children,
    selectedItemChanged,
    openMenu,
    compactMode,
    onMenuReady,
  } = props;

  const { isLarge } = useScreenSize();
  function normalizePath() {
    return navigation?.map((item: { path: string; icon: string }) => ({
      ...item,
      expanded: isLarge,
      path: item.path && !/^\//.test(item.path) ? `/${item.path}` : item.path,
      icon: getCustomIcon(item?.icon)?.src,
    }));
  }

  const items: SideNavigationItem[] = useMemo(normalizePath, [
    isLarge,
    navigation,
  ]);

  const { navigationData: { currentPath } = {} } = useNavigation();

  const treeViewRef = useRef<TreeView>(null);
  const wrapperRef: any = useRef();
  const getWrapperRef = useCallback(
    (element: Element | Element[] | any) => {
      const prevElement = wrapperRef.current;
      if (prevElement) {
        events.off(prevElement, "dxclick");
      }

      wrapperRef.current = element;
      events.on(element, "dxclick", (e: React.PointerEvent) => {
        openMenu(e);
      });
    },
    [openMenu]
  );

  useEffect(() => {
    const treeView = treeViewRef.current && treeViewRef.current.instance;
    if (!treeView) {
      return;
    }

    if (currentPath !== undefined) {
      treeView.selectItem(currentPath);
      treeView.expandItem(currentPath);
    }

    if (compactMode) {
      treeView.collapseAll();
    }
  }, [currentPath, compactMode]);

  return (
    <div
      className="dx-swatch-additional side-navigation-menu"
      ref={getWrapperRef}
    >
      {children}
      <div className="menu-container theme-dependent">
        <TreeView
          ref={treeViewRef}
          items={items}
          keyExpr="path"
          selectionMode="single"
          focusStateEnabled={false}
          expandEvent="click"
          onItemClick={selectedItemChanged}
          onContentReady={onMenuReady}
          width="100%"
        />
      </div>
    </div>
  );
};
